/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/* $Source: src/include/usr/hwpf/fapi/fapiReturnCodes.H $                 */
/*                                                                        */
/* OpenPOWER HostBoot Project                                             */
/*                                                                        */
/* COPYRIGHT International Business Machines Corp. 2011,2014              */
/*                                                                        */
/* Licensed under the Apache License, Version 2.0 (the "License");        */
/* you may not use this file except in compliance with the License.       */
/* You may obtain a copy of the License at                                */
/*                                                                        */
/*     http://www.apache.org/licenses/LICENSE-2.0                         */
/*                                                                        */
/* Unless required by applicable law or agreed to in writing, software    */
/* distributed under the License is distributed on an "AS IS" BASIS,      */
/* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or        */
/* implied. See the License for the specific language governing           */
/* permissions and limitations under the License.                         */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
// $Id: fapiReturnCodes.H,v 1.17 2014/04/15 17:50:40 whs Exp $
// $Source: /afs/awd/projects/eclipz/KnowledgeBase/.cvsroot/eclipz/hwpf/working/fapi/fapiReturnCodes.H,v $

/**
 *  @file fapiReturnCodes.H
 *
 *  @brief Defines the returns codes generated by HWPF code.
 */

/*
 * Change Log ******************************************************************
 * Flag     Defect/Feature  User        Date        Description
 * ------   --------------  ----------  ----------- ----------------------------
 *                          mjjones     04/13/2011  Created.
 *                          mjjones     06/07/2011  Add FAPI_RC_NOT_IMPLEMENTED
 *                          mjjones     06/30/2011  Add FAPI_RC_ATTR_UNIT_TEST_FAIL
 *                          mjjones     08/10/2011  Add FAPI_RC_FFDC_HWP_NOT_FOUND
 *                          mjjones     09/23/2011  Remove FAPI_RC_FFDC_HWP_NOT_FOUND
 *                          mjjones     10/07/2011  Add FAPI_RC_INVALID_ATTR_GET
 *                          mjjones     10/28/2011  Trim FAPI/PLAT errors
 *                          mjjones     11/10/2011  Use ecmdDataBufferBase
 *                          mjjones     01/13/2012  Name enumeration
 *                          mjjones     04/16/2012  Add FAPI_RC_INVALID_CHIP_EC_FEATURE_GET
 *                          mjjones     05/02/2012  Add FAPI_RC_PLAT_ERR_ADU_LOCKED
 *                          mjjones     11/14/2012  Add FAPI_RC_UNRECOGNIZED_SBE_ERROR
 *                          mjjones     03/26/2014  Remove FAPI_RC_UNRECOGNIZED_SBE_ERROR
 *                          whs         04/14/2014  Remove FAPI_RC_INVALID_PARAM
 */

#ifndef FAPIRETURNCODES_H_
#define FAPIRETURNCODES_H_

#include <ecmdDataBufferBase.H>

namespace fapi
{

/**
 * @brief Enumeration of return codes
 *
 * @note HWP return codes are in the HwpReturnCode enumeration in the
 *       generated fapiHwpReturnCodes.H file
 */
enum ReturnCodes
{
    FAPI_RC_SUCCESS = 0,

    //--------------------------------------------------------------------------
    // Flag bits indicating which code generated the error If no flag set then
    // it is generated by HWP
    //--------------------------------------------------------------------------
    FAPI_RC_FAPI_MASK = 0x04000000, // FAPI generated error
    FAPI_RC_PLAT_MASK = 0x02000000, // PLAT generated error
    FAPI_RC_ECMD_MASK = ECMD_ERR_ECMD, // ECMD generated error (0x01000000)

    //--------------------------------------------------------------------------
    // FAPI generated return codes
    //--------------------------------------------------------------------------

    // Initfile requested an attribute with an invalid attribute ID
    FAPI_RC_INVALID_ATTR_GET            = FAPI_RC_FAPI_MASK | 0x01,

    // HWP requested a chip EC feature with an invalid attribute ID
    FAPI_RC_INVALID_CHIP_EC_FEATURE_GET = FAPI_RC_FAPI_MASK | 0x02,

    // Invalid multiscom parameters
    FAPI_RC_INVALID_MULTISCOM_LENGTH    = FAPI_RC_FAPI_MASK | 0x03,

    //--------------------------------------------------------------------------
    // PLAT generated return codes. Additional details may be contained in
    // ReturnCode platData (this can only be looked at by PLAT code)
    //--------------------------------------------------------------------------

    // Generic platform error
    FAPI_RC_PLAT_ERR_SEE_DATA   = FAPI_RC_PLAT_MASK | 0x01,

    // Operation to AlterDisplay unit failed because it is locked
    FAPI_RC_PLAT_ERR_ADU_LOCKED = FAPI_RC_PLAT_MASK | 0x02,

    // Operation not supported by HB runtime
    FAPI_RC_PLAT_NOT_SUPPORTED_AT_RUNTIME = FAPI_RC_PLAT_MASK | 0x03,
};

}

#endif // FAPIRETURNCODES_H_
